CONFIG = dict(

    default_flags = ['-o'],
    # For default flags. Right now, it organizes your files into an
    # artist/album/song structure.
    # To add a flag or argument, add an element to the index:
    # default_flags = ['-o', '-l', '~/Music']

    SPOTIFY_CLIENT_ID = '',
    SPOTIFY_CLIENT_SECRET = '',
    # You can either specify Spotify keys here, or in environment variables.

    additional_search_terms = 'lyrics',
    # Search terms for youtube

    organize = True,
    # True always forces organization.
    # False always forces non-organization.
    # None allows options and flags to determine if the files
    # will be organized.

    custom_directory = "",
    # Defaults to '~/Music'
)
